@@ -55,7 +55,8 @@ KNOWN_RESOURCES = (
     'ports',
     'routers',
     'subnets',
-    'subnetpools'
+    'subnetpools',
+    'security_groups'
 )
 
 KNOWN_HTTP_ACTIONS = (
@@ -102,6 +103,7 @@ KNOWN_EXTENSIONS = (
     'fip-port-details',
     'flavors',
     'floating-ip-port-forwarding',
+    'floating-ip-port-forwarding-description',
     'floatingip-autodelete-internal',
     'floatingip-pools',
     'ip-substring-filtering',
@@ -111,6 +113,7 @@ KNOWN_EXTENSIONS = (
     'l3-port-ip-change-not-allowed',
     'logging',
     'metering',
+    'metering_source_and_destination_filters',
     'multi-provider',
     'net-mtu',
     'network-ip-availability',
@@ -124,10 +127,13 @@ KNOWN_EXTENSIONS = (
     'qos',
     'qos-bw-limit-direction',
     'qos-gateway-ip',
+    'qos-port-network-policy',
     'qos-rules-alias',
     'quotas',
+    'rbac-address-scope',
     'rbac-policies',
     'rbac-security-groups',
+    'rbac-subnetpool',
     'router',
     'router_availability_zone',
     'security-group',
@@ -168,9 +174,6 @@ KNOWN_EXTENSIONS = (
     # https://opendev.org/openstack/networking-sfc:
     'flow_classifier',
     'sfc',
-
-    # https://opendev.org/openstack/neutron-interconnection:
-    'interconnection',
 )
 
 KNOWN_KEYWORDS = (
