@@ -16,11 +16,13 @@ import inspect
 import re
 
 import netaddr
+from os_ken.lib.packet import ether_types
+from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
 from oslo_utils import strutils
 from oslo_utils import uuidutils
-import six
+from webob import exc
 
 from neutron_lib._i18n import _
 from neutron_lib import constants
@@ -161,13 +163,13 @@ def validate_values(data, valid_values=None, valid_values_display=None):
                 LOG.debug(msg)
                 return msg
         except TypeError:
-                # This is a programming error
-                msg = (_("'data' of type '%(typedata)s' and 'valid_values'"
-                         "of type '%(typevalues)s' are not "
-                         "compatible for comparison") %
-                       {'typedata': type(data),
-                        'typevalues': type(valid_values)})
-                raise TypeError(msg)
+            # This is a programming error
+            msg = (_("'data' of type '%(typedata)s' and 'valid_values' "
+                     "of type '%(typevalues)s' are not "
+                     "compatible for comparison") %
+                   {'typedata': type(data),
+                    'typevalues': type(valid_values)})
+            raise TypeError(msg)
     else:
         # This is a programming error
         msg = (_("'valid_values' does not support membership operations"))
@@ -225,7 +227,7 @@ def validate_string(data, max_len=None):
         the given max_len. Otherwise a human readable message indicating why
         the data is invalid.
     """
-    if not isinstance(data, six.string_types):
+    if not isinstance(data, str):
         msg = _("'%s' is not a valid string") % data
         LOG.debug(msg)
         return msg
@@ -599,7 +601,7 @@ def validate_hostroutes(data, valid_values=None):
         msg = _verify_dict_keys(expected_keys, hostroute)
         if msg:
             return msg
-        msg = validate_subnet(hostroute['destination'])
+        msg = validate_route_cidr(hostroute['destination'])
         if msg:
             return msg
         msg = validate_ip_address(hostroute['nexthop'])
@@ -664,6 +666,32 @@ def validate_subnet(data, valid_values=None):
     return msg
 
 
+def validate_route_cidr(data, valid_values=None):
+    """Validate data is a proper CIDR string.
+
+    :param data: The data to validate.
+    :param valid_values: Not used!
+    :returns: None if data is valid CIDR. Otherwise a human
+              readable message as to why data is invalid.
+    """
+    msg = None
+    try:
+        net = netaddr.IPNetwork(validate_no_whitespace(data))
+        if '/' not in data or (net.network != net.ip):
+            msg = _("'%(data)s' is not a recognized CIDR,"
+                    " '%(cidr)s' is recommended") % {"data": data,
+                                                     "cidr": net.cidr}
+        elif net.is_loopback():
+            msg = _("'%(data)s' is not a routable CIDR") % {"data": data}
+        else:
+            return
+    except Exception:
+        msg = _("'%s' is not a valid CIDR") % data
+    if msg:
+        LOG.debug(msg)
+    return msg
+
+
 def validate_subnet_or_none(data, valid_values=None):
     """Validate data is a valid subnet address string or None.
 
@@ -1077,11 +1105,77 @@ def validate_service_plugin_type(data, valid_values=None):
         raise n_exc.InvalidServiceType(service_type=data)
 
 
+def validate_subnet_service_types(service_types, valid_values=None):
+    if service_types:
+        if not isinstance(service_types, list):
+            raise exc.HTTPBadRequest(
+                _("Subnet service types must be a list."))
+
+        # Include standard prefixes
+        prefixes = list(constants.DEVICE_OWNER_PREFIXES)
+        prefixes += constants.DEVICE_OWNER_COMPUTE_PREFIX
+
+        for service_type in service_types:
+            if not isinstance(service_type, str):
+                raise n_exc.InvalidInputSubnetServiceType(
+                    service_type=service_type)
+            elif not service_type.startswith(tuple(prefixes)):
+                raise n_exc.InvalidSubnetServiceType(service_type=service_type)
+
+
+def validate_ethertype(ethertype, valid_values=None):
+    """Validates ethertype is a valid ethertype.
+
+    :param ethertype: Ethertype to validate.
+    :returns: None if data is a valid ethertype.  Otherwise a human-readable
+        message indicating that the data is not a valid ethertype.
+    """
+
+    if cfg.CONF.sg_filter_ethertypes:
+        os_ken_ethertypes = ether_types.__dict__
+        try:
+            # Use the ethertype constants from os_ken with the special
+            # exception of 'IPv4', which appears as 'ETH_TYPE_IP' in os_ken.
+            # Be case-insensitive.
+            ethertype_str = str(ethertype).upper()
+            if ethertype_str == "IPV4":
+                ethertype_str = "IP"
+            ethertype_name = 'ETH_TYPE_' + ethertype_str
+            if ethertype_name in os_ken_ethertypes:
+                ethertype = os_ken_ethertypes[ethertype_name]
+        except TypeError:
+            # Value of ethertype cannot be coerced into a string, like None
+            pass
+        try:
+            if isinstance(ethertype, str):
+                ethertype = int(ethertype, 16)
+            if (isinstance(ethertype, int) and
+                    constants.ETHERTYPE_MIN <= ethertype and
+                    ethertype <= constants.ETHERTYPE_MAX):
+                return None
+        except ValueError:
+            pass
+        msg = _("Ethertype %s is not a two octet hexadecimal "
+                "number or ethertype name.") % ethertype
+        LOG.debug(msg)
+        return msg
+    else:
+        if ethertype in constants.VALID_ETHERTYPES:
+            return None
+        valids = ','.join(map(str, constants.VALID_ETHERTYPES))
+        msg = _("Ethertype %(ethertype)s is not a valid ethertype, must be "
+                "one of %(valid_ethertypes)s.") % {'ethertype': ethertype,
+                                                   'valid_ethertypes': valids}
+        LOG.debug(msg)
+        return msg
+
+
 # Dictionary that maintains a list of validation functions
 validators = {'type:dict': validate_dict,
               'type:dict_or_none': validate_dict_or_none,
               'type:dict_or_empty': validate_dict_or_empty,
               'type:dict_or_nodata': validate_dict_or_nodata,
+              'type:ethertype': validate_ethertype,
               'type:fixed_ips': validate_fixed_ips,
               'type:hostroutes': validate_hostroutes,
               'type:ip_address': validate_ip_address,
@@ -1120,7 +1214,9 @@ validators = {'type:dict': validate_dict,
               'type:list_of_any_key_specs_or_none':
                   validate_any_key_specs_or_none,
               'type:service_plugin_type': validate_service_plugin_type,
-              'type:list_of_subnets_or_none': validate_subnet_list_or_none
+              'type:list_of_subnets_or_none': validate_subnet_list_or_none,
+              'type:list_of_subnet_service_types':
+                  validate_subnet_service_types
               }
 
 
