@@ -15,12 +15,17 @@
 
 import functools
 import re
+import time
+from urllib import parse
+import uuid
+
+import requests
 
 from keystoneauth1 import exceptions as ks_exc
 from keystoneauth1 import loading as keystone
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import versionutils
-from six.moves.urllib.parse import urlencode
 
 from neutron_lib._i18n import _
 from neutron_lib.exceptions import placement as n_exc
@@ -32,7 +37,10 @@ API_VERSION_REQUEST_HEADER = 'OpenStack-API-Version'
 PLACEMENT_API_WITH_MEMBER_OF = 'placement 1.3'
 PLACEMENT_API_WITH_NESTED_RESOURCES = 'placement 1.14'
 PLACEMENT_API_RETURN_PROVIDER_BODY = 'placement 1.20'
-PLACEMENT_API_LATEST_SUPPORTED = PLACEMENT_API_RETURN_PROVIDER_BODY
+PLACEMENT_API_ERROR_CODE = 'placement 1.23'
+PLACEMENT_API_CONSUMER_GENERATION = 'placement 1.28'
+PLACEMENT_API_LATEST_SUPPORTED = PLACEMENT_API_CONSUMER_GENERATION
+GENERATION_CONFLICT_RETRIES = 10
 
 
 def _check_placement_api_available(f):
@@ -67,6 +75,73 @@ def _get_version(openstack_api_version):
     return versionutils.convert_version_to_tuple(match.group('api_version'))
 
 
+class UUIDEncoder(jsonutils.JSONEncoder):
+    def default(self, o):
+        if isinstance(o, uuid.UUID):
+            return str(o)
+        return super(UUIDEncoder, self).default(o)
+
+
+class NoAuthClient(object):
+    """Placement NoAuthClient for fullstack testing"""
+
+    def __init__(self, url):
+        self.url = url
+        # TODO(lajoskatona): use perhaps http_connect_timeout from
+        # keystone_authtoken group
+        self.timeout = 5
+        self.retries = 2
+
+    def request(self, url, method, body=None, headers=None, **kwargs):
+        headers = headers or {}
+        headers.setdefault('Accept', 'application/json')
+
+        # Note(lajoskatona): placement report plugin fills uuid fields with
+        # UUID objects, and that is good for keystone, but requests goes mad
+        # with that if we use json=body as it can't serialize UUID.
+        # To make things worse if we give a serialized json, it will do the
+        # jsonification again, so better to create the json here and give it
+        # to requests with the data parameter.
+        body = jsonutils.dumps(body, cls=UUIDEncoder)
+        for i in range(self.retries):
+            try:
+                resp = requests.request(
+                    method,
+                    url,
+                    data=body,
+                    headers=headers,
+                    verify=False,
+                    timeout=self.timeout,
+                    **kwargs)
+                return resp
+            except requests.Timeout:
+                LOG.exception('requests Timeout, let\'s retry it...')
+            except requests.ConnectionError:
+                LOG.exception('Connection Error appeared')
+            except requests.RequestException:
+                LOG.exception('Some really weird thing happened, let\'s '
+                              'retry it')
+            time.sleep(self.timeout)
+        # Note(lajoskatona): requests raise ConnectionError, but
+        # PlacementReportPlugin expects keystonauth1 HttpError.
+        raise ks_exc.HttpError
+
+    def get(self, url, endpoint_filter, **kwargs):
+        return self.request('%s%s' % (self.url, url), 'GET', **kwargs)
+
+    def post(self, url, json, endpoint_filter, **kwargs):
+        return self.request('%s%s' % (self.url, url), 'POST', body=json,
+                            **kwargs)
+
+    def put(self, url, json, endpoint_filter, **kwargs):
+        resp = self.request('%s%s' % (self.url, url), 'PUT', body=json,
+                            **kwargs)
+        return resp
+
+    def delete(self, url, endpoint_filter, **kwargs):
+        return self.request('%s%s' % (self.url, url), 'DELETE', **kwargs)
+
+
 class PlacementAPIClient(object):
     """Client class for placement ReST API."""
 
@@ -76,7 +151,8 @@ class PlacementAPIClient(object):
         self._target_version = _get_version(openstack_api_version)
         self._conf = conf
         self._ks_filter = {'service_type': 'placement',
-                           'region_name': self._conf.placement.region_name}
+                           'region_name': self._conf.placement.region_name,
+                           'interface': self._conf.placement.endpoint_type}
         self._api_version_header = {API_VERSION_REQUEST_HEADER:
                                     self._openstack_api_version}
         self._client = None
@@ -87,11 +163,18 @@ class PlacementAPIClient(object):
         # clean slate.
         self._resource_providers = {}
         self._provider_aggregate_map = {}
-        auth_plugin = keystone.load_auth_from_conf_options(
-            self._conf, 'placement')
-        return keystone.load_session_from_conf_options(
-            self._conf, 'placement', auth=auth_plugin,
-            additional_headers={'accept': 'application/json'})
+        # TODO(lajoskatona): perhaps not the best to override config options,
+        # actually the abused keystoneauth1 options are:
+        # auth_type (used for deciding for NoAuthClient) and auth_section
+        # (used for communicating the url for the NoAuthClient)
+        if self._conf.placement.auth_type == 'noauth':
+            return NoAuthClient(self._conf.placement.auth_section)
+        else:
+            auth_plugin = keystone.load_auth_from_conf_options(
+                self._conf, 'placement')
+            return keystone.load_session_from_conf_options(
+                self._conf, 'placement', auth=auth_plugin,
+                additional_headers={'accept': 'application/json'})
 
     def _extend_header_with_api_version(self, **kwargs):
         headers = kwargs.get('headers', {})
@@ -117,6 +200,46 @@ class PlacementAPIClient(object):
         return self._client.put(url, json=data,
                                 endpoint_filter=self._ks_filter, **kwargs)
 
+    def _put_with_retry_for_generation_conflict(
+            self, url, body,
+            resource_provider_uuid,
+            resource_provider_generation=None):
+
+        if resource_provider_generation is None:
+            # If the client's user did not supply a generation to us we dare to
+            # retry without handing the control back to our caller.
+            max_tries = GENERATION_CONFLICT_RETRIES
+        else:
+            # If the client's user supplied a generation to us we don't dare to
+            # retry on her behalf since we don't know her intention.
+            max_tries = 1
+
+        body['resource_provider_generation'] = resource_provider_generation
+
+        for i in range(max_tries):
+            if resource_provider_generation is None:
+                # In the bodies of
+                # PUT /resource_providers/{uuid}/traits
+                # PUT /resource_providers/{uuid}/inventories
+                # PUT /resource_providers/{uuid}/inventories/{resource_class}
+                # resource_provider_generation happens to be at the same place.
+                body['resource_provider_generation'] = \
+                    self.get_resource_provider(
+                        resource_provider_uuid=resource_provider_uuid)[
+                            'generation']
+            try:
+                return self._put(url, body).json()
+            except ks_exc.Conflict as e:
+                if e.response.json()[
+                        'errors'][0]['code'] == 'placement.concurrent_update':
+                    continue
+                else:
+                    raise
+
+        raise n_exc.PlacementResourceProviderGenerationConflict(
+            resource_provider=resource_provider_uuid,
+            generation=body['resource_provider_generation'])
+
     def _delete(self, url, **kwargs):
         kwargs = self._extend_header_with_api_version(**kwargs)
         return self._client.delete(url, endpoint_filter=self._ks_filter,
@@ -133,7 +256,12 @@ class PlacementAPIClient(object):
         :returns: The resource provider created.
         """
         url = '/resource_providers'
-        return self._post(url, resource_provider).json()
+        rsp = self._post(url, resource_provider)
+        if (self._target_version <
+                _get_version(PLACEMENT_API_RETURN_PROVIDER_BODY)):
+            return
+        else:
+            return rsp.json()
 
     @_check_placement_api_available
     def update_resource_provider(self, resource_provider):
@@ -248,7 +376,7 @@ class PlacementAPIClient(object):
             filters['in_tree'] = in_tree
         if uuid:
             filters['uuid'] = uuid
-        url = '%s?%s' % (url, urlencode(filters))
+        url = '%s?%s' % (url, parse.urlencode(filters))
         return self._get(url).json()
 
     @_check_placement_api_available
@@ -279,23 +407,19 @@ class PlacementAPIClient(object):
                                                              server side.
         :returns: The updated set of inventory records.
         """
-        if resource_provider_generation is None:
-            resource_provider_generation = self.get_resource_provider(
-                resource_provider_uuid=resource_provider_uuid)['generation']
         url = '/resource_providers/%s/inventories' % resource_provider_uuid
         body = {
             'resource_provider_generation': resource_provider_generation,
             'inventories': inventories
         }
+
         try:
-            return self._put(url, body).json()
+            return self._put_with_retry_for_generation_conflict(
+                url, body, resource_provider_uuid,
+                resource_provider_generation)
         except ks_exc.NotFound:
             raise n_exc.PlacementResourceProviderNotFound(
                 resource_provider=resource_provider_uuid)
-        except ks_exc.Conflict:
-            raise n_exc.PlacementResourceProviderGenerationConflict(
-                resource_provider=resource_provider_uuid,
-                generation=resource_provider_generation)
 
     @_check_placement_api_available
     def delete_resource_provider_inventories(self, resource_provider_uuid):
@@ -385,26 +509,23 @@ class PlacementAPIClient(object):
                                              provider. Optional.
         :raises PlacementResourceNotFound: If the resource provider or the
                                            resource class is not found.
-        :raises PlacementInventoryUpdateConflict: If the resource provider
-                                                  generation does not match
-                                                  with the server side.
+        :raises PlacementResourceProviderGenerationConflict: If the resource
+                                                             provider
+                                                             generation does
+                                                             not match with the
+                                                             server side.
         :returns: The updated inventory of the resource class as a dict.
         """
-        if resource_provider_generation is None:
-            resource_provider_generation = self.get_resource_provider(
-                resource_provider_uuid=resource_provider_uuid)['generation']
         url = '/resource_providers/%s/inventories/%s' % (
             resource_provider_uuid, resource_class)
-        inventory['resource_provider_generation'] = \
-            resource_provider_generation
+        body = inventory
+
         try:
-            return self._put(url, inventory).json()
+            return self._put_with_retry_for_generation_conflict(
+                url, body, resource_provider_uuid,
+                resource_provider_generation)
         except ks_exc.NotFound as e:
             raise n_exc.PlacementResourceNotFound(url=e.url)
-        except ks_exc.Conflict:
-            raise n_exc.PlacementInventoryUpdateConflict(
-                resource_provider=resource_provider_uuid,
-                resource_class=resource_class)
 
     @_check_placement_api_available
     def associate_aggregates(self, resource_provider_uuid, aggregates):
@@ -489,24 +610,32 @@ class PlacementAPIClient(object):
                                        to set the traits
         :param traits: a list of traits.
         :param resource_provider_generation: The generation of the resource
-                                             provider. Optional.
+                                             provider. Optional. If not
+                                             supplied by the caller, handle
+                                             potential generation conflict
+                                             by retrying the call. If supplied
+                                             we assume the caller handles
+                                             generation conflict.
         :raises PlacementResourceProviderNotFound: If the resource provider
                                                    is not found.
         :raises PlacementTraitNotFound: If any of the specified traits are not
                                         valid.
+        :raises PlacementResourceProviderGenerationConflict: For concurrent
+                                                             conflicting
+                                                             updates detected.
         :returns: The new traits of the resource provider together with the
                   resource provider generation.
         """
-        if resource_provider_generation is None:
-            resource_provider_generation = self.get_resource_provider(
-                resource_provider_uuid=resource_provider_uuid)['generation']
         url = '/resource_providers/%s/traits' % (resource_provider_uuid)
         body = {
             'resource_provider_generation': resource_provider_generation,
             'traits': traits
         }
+
         try:
-            return self._put(url, body).json()
+            return self._put_with_retry_for_generation_conflict(
+                url, body, resource_provider_uuid,
+                resource_provider_generation)
         except ks_exc.NotFound:
             raise n_exc.PlacementResourceProviderNotFound(
                 resource_provider=resource_provider_uuid)
@@ -604,3 +733,69 @@ class PlacementAPIClient(object):
             self._delete(url)
         except ks_exc.NotFound:
             raise n_exc.PlacementResourceClassNotFound(resource_class=name)
+
+    @_check_placement_api_available
+    def list_allocations(self, consumer_uuid):
+        """List allocations for the consumer
+
+        :param consumer_uuid: The uuid of the consumer, in case of bound port
+                              owned by a VM, the VM uuid.
+        :returns: All allocation records for the consumer.
+        """
+        url = '/allocations/%s' % consumer_uuid
+        return self._get(url).json()
+
+    def update_qos_minbw_allocation(self, consumer_uuid, minbw_alloc_diff,
+                                    rp_uuid):
+        """Update allocation for QoS minimum bandwidth consumer
+
+        :param consumer_uuid: The uuid of the consumer, in case of bound port
+                              owned by a VM, the VM uuid.
+        :param minbw_alloc_diff: A dict which contains the fields to update
+                            for the allocation under the given resource
+                            provider.
+        :param rp_uuid: uuid of the resource provider for which the
+                        allocations are to be updated.
+        """
+        for i in range(GENERATION_CONFLICT_RETRIES):
+            body = self.list_allocations(consumer_uuid)
+            if not body['allocations']:
+                raise n_exc.PlacementAllocationRemoved(consumer=consumer_uuid)
+            if rp_uuid not in body['allocations']:
+                raise n_exc.PlacementAllocationRpNotExists(
+                    resource_provider=rp_uuid, consumer=consumer_uuid)
+            # Count new min_kbps values based on the diff in alloc_diff
+            for drctn, min_kbps_diff in minbw_alloc_diff.items():
+                orig_kbps = body['allocations'][rp_uuid]['resources'][drctn]
+                new_kbps = orig_kbps + min_kbps_diff
+                if new_kbps > 0:
+                    body['allocations'][rp_uuid]['resources'][drctn] = new_kbps
+                else:
+                    # Remove the resource class if the new value for min_kbps
+                    # is 0
+                    resources = body['allocations'][rp_uuid]['resources']
+                    if len(resources) > 1:
+                        resources.pop(drctn, None)
+                    else:
+                        body['allocations'].pop(rp_uuid)
+            try:
+                # Update allocations has no return body, but leave the loop
+                return self.update_allocation(consumer_uuid, body)
+            except ks_exc.Conflict as e:
+                resp = e.response.json()
+                if resp['errors'][0]['code'] == 'placement.concurrent_update':
+                    continue
+                else:
+                    raise
+        raise n_exc.PlacementAllocationGenerationConflict(
+            consumer=consumer_uuid)
+
+    def update_allocation(self, consumer_uuid, allocations):
+        """Update allocation record for given consumer and rp
+
+        :param consumer_uuid: The uuid of the consumer
+        :param allocations: Dict in the form described in placement API ref:
+                            https://tinyurl.com/yxeuzn6l
+        """
+        url = '/allocations/%s' % consumer_uuid
+        self._put(url, allocations)
