@@ -14,8 +14,6 @@
 
 import abc
 
-import six
-
 
 # The following keys are used in the segment dictionaries passed via
 # the driver API.
@@ -33,8 +31,7 @@ BOUND_DRIVER = 'bound_driver'
 BOUND_SEGMENT = 'bound_segment'
 
 
-@six.add_metaclass(abc.ABCMeta)
-class MechanismDriver(object):
+class MechanismDriver(object, metaclass=abc.ABCMeta):
     """Define stable abstract interface for ML2 mechanism drivers.
 
     A mechanism driver is called on the creation, update, and deletion
@@ -447,9 +444,34 @@ class MechanismDriver(object):
         """
         return False
 
+    @staticmethod
+    def provider_network_attribute_updates_supported():
+        """Returns the provider network attributes that can be updated
 
-@six.add_metaclass(abc.ABCMeta)
-class _TypeDriverBase(object):
+        Possible values: neutron_lib.api.definitions.provider_net.ATTRIBUTES
+
+        :returns: (list) provider network attributes that can be updated in a
+                         live network using this driver.
+        """
+        return []
+
+    def supported_extensions(self, extensions):
+        """Return the mechanism driver supported extensions
+
+        By default this method will return the same provided set, without any
+        filtering. In case any particular mechanism driver needs to filter out
+        any specific extension or supports only a reduced set of extensions,
+        this method should be override.
+
+        :param extensions: set of extensions supported by the instance that
+                           created this mechanism driver.
+        :returns: a set of the extensions currently supported by this
+                  mechanism driver
+        """
+        return extensions
+
+
+class _TypeDriverBase(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_type(self):
@@ -509,8 +531,7 @@ class _TypeDriverBase(object):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class TypeDriver(_TypeDriverBase):
+class TypeDriver(_TypeDriverBase, metaclass=abc.ABCMeta):
     """Define abstract interface for ML2 type drivers.
 
     ML2 type drivers each support a specific network_type for provider
@@ -581,8 +602,7 @@ class TypeDriver(_TypeDriverBase):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ML2TypeDriver(_TypeDriverBase):
+class ML2TypeDriver(_TypeDriverBase, metaclass=abc.ABCMeta):
     """Define abstract interface for ML2 type drivers.
 
     ML2 type drivers each support a specific network_type for provider
@@ -652,9 +672,28 @@ class ML2TypeDriver(_TypeDriverBase):
         """
         pass
 
+    @abc.abstractmethod
+    def initialize_network_segment_range_support(self):
+        """Perform driver network segment range initialization.
 
-@six.add_metaclass(abc.ABCMeta)
-class NetworkContext(object):
+        Called during the initialization of the ``network-segment-range``
+        service plugin if enabled, after all drivers have been loaded and the
+        database has been initialized. This reloads the `default`
+        network segment ranges when Neutron server starts/restarts.
+        """
+        pass
+
+    @abc.abstractmethod
+    def update_network_segment_range_allocations(self):
+        """Update driver network segment range allocations.
+
+        This syncs the driver segment allocations when network segment ranges
+        have been created, updated or deleted.
+        """
+        pass
+
+
+class NetworkContext(object, metaclass=abc.ABCMeta):
     """Context passed to MechanismDrivers for changes to network resources.
 
     A NetworkContext instance wraps a network resource. It provides
@@ -691,8 +730,7 @@ class NetworkContext(object):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class SubnetContext(object):
+class SubnetContext(object, metaclass=abc.ABCMeta):
     """Context passed to MechanismDrivers for changes to subnet resources.
 
     A SubnetContext instance wraps a subnet resource. It provides
@@ -724,8 +762,7 @@ class SubnetContext(object):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PortContext(object):
+class PortContext(object, metaclass=abc.ABCMeta):
     """Context passed to MechanismDrivers for changes to port resources.
 
     A PortContext instance wraps a port resource. It provides helper
@@ -1042,8 +1079,7 @@ class PortContext(object):
         pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ExtensionDriver(object):
+class ExtensionDriver(object, metaclass=abc.ABCMeta):
     """Define stable abstract interface for ML2 extension drivers.
 
     An extension driver extends the core resources implemented by the
