@@ -10,8 +10,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import re
-
 import testtools
 
 from neutron_lib.hacking import checks
@@ -28,10 +26,19 @@ class HackingTestCase(base.BaseTestCase):
     def assertLineFails(self, func, *args):
         self.assertIsInstance(next(func(*args)), tuple)
 
+    def _get_factory_checks(self, factory):
+        check_fns = []
+
+        def _reg(check_fn):
+            self.assertTrue(hasattr(check_fn, '__call__'))
+            self.assertFalse(check_fn in check_fns)
+            check_fns.append(check_fn)
+
+        factory(_reg)
+        return check_fns
+
     def test_factory(self):
-        def check_callable(fn):
-            self.assertTrue(hasattr(fn, '__call__'))
-        self.assertIsNone(checks.factory(check_callable))
+        self.assertGreater(len(self._get_factory_checks(checks.factory)), 0)
 
     def test_use_jsonutils(self):
         def __get_msg(fun):
@@ -62,14 +69,6 @@ class HackingTestCase(base.BaseTestCase):
                     "./neutron/plugins/ml2/drivers/openvswitch/agent/xenapi/"
                     "etc/xapi.d/plugins/netwrap"))))
 
-    def test_check_oslo_namespace_imports(self):
-        f = checks.check_oslo_namespace_imports
-        self.assertLinePasses(f, 'from oslo_utils import importutils')
-        self.assertLinePasses(f, 'import oslo_messaging')
-        self.assertLineFails(f, 'from oslo.utils import importutils')
-        self.assertLineFails(f, 'from oslo import messaging')
-        self.assertLineFails(f, 'import oslo.messaging')
-
     def test_check_contextlib_nested(self):
         f = checks.check_no_contextlib_nested
         self.assertLineFails(f, 'with contextlib.nested():', '')
@@ -77,24 +76,6 @@ class HackingTestCase(base.BaseTestCase):
         self.assertLinePasses(f, '# with contextlib.nested():', '')
         self.assertLinePasses(f, 'print("with contextlib.nested():")', '')
 
-    def test_check_python3_xrange(self):
-        f = checks.check_python3_xrange
-        self.assertLineFails(f, 'a = xrange(1000)')
-        self.assertLineFails(f, 'b =xrange   (   42 )')
-        self.assertLineFails(f, 'c = xrange(1, 10, 2)')
-        self.assertLinePasses(f, 'd = range(1000)')
-        self.assertLinePasses(f, 'e = six.moves.range(1337)')
-
-    def test_no_basestring(self):
-        f = checks.check_no_basestring
-        self.assertLineFails(f, 'isinstance(x, basestring)')
-        self.assertLinePasses(f, 'isinstance(x, BaseString)')
-
-    def test_check_python3_iteritems(self):
-        f = checks.check_python3_no_iteritems
-        self.assertLineFails(f, "d.iteritems()")
-        self.assertLinePasses(f, "six.iteritems(d)")
-
     def test_no_mutable_default_args(self):
         self.assertEqual(1, len(list(checks.no_mutable_default_args(
             " def fake_suds_context(calls={}):"))))
@@ -116,52 +97,24 @@ class HackingTestCase(base.BaseTestCase):
         self.assertLineFails(f, 'from neutron import context')
         self.assertLineFails(f, 'import neutron.common.config')
 
-    def test_log_translations(self):
-        expected_marks = {
-            'error': '_LE',
-            'info': '_LI',
-            'warning': '_LW',
-            'critical': '_LC',
-            'exception': '_LE',
-        }
-        logs = expected_marks.keys()
-        debug = "LOG.debug('OK')"
-        self.assertEqual(
-            0, len(list(tc.validate_log_translations(debug, debug, 'f'))))
-        for log in logs:
-            bad = 'LOG.%s(_("Bad"))' % log
-            self.assertEqual(
-                1, len(list(tc.validate_log_translations(bad, bad, 'f'))))
-            bad = 'LOG.%s("Bad")' % log
-            self.assertEqual(
-                1, len(list(tc.validate_log_translations(bad, bad, 'f'))))
-            ok = "LOG.%s('OK')    # noqa" % log
-            self.assertEqual(
-                0, len(list(tc.validate_log_translations(ok, ok, 'f'))))
-            ok = "LOG.%s(variable)" % log
-            self.assertEqual(
-                0, len(list(tc.validate_log_translations(ok, ok, 'f'))))
-            # Do not do validations in tests
-            ok = 'LOG.%s("OK - unit tests")' % log
-            self.assertEqual(
-                0, len(list(tc.validate_log_translations(ok, ok,
-                                                         'f/tests/f'))))
-
-            for mark in tc._all_hints:
-                stmt = "LOG.%s(%s('test'))" % (log, mark)
+    def test_no_log_translations(self):
+        for log in tc._all_log_levels:
+            for hint in tc._all_hints:
+                bad = 'LOG.%s(%s("Bad"))' % (log, hint)
                 self.assertEqual(
-                    0 if expected_marks[log] == mark else 1,
-                    len(list(tc.validate_log_translations(stmt, stmt, 'f'))))
-
-    def test_no_translate_debug_logs(self):
-        for hint in tc._all_hints:
-            bad = "LOG.debug(%s('bad'))" % hint
-            self.assertEqual(
-                1, len(list(tc.no_translate_debug_logs(bad, 'f'))))
+                    1, len(list(tc.no_translate_logs(bad, 'f'))))
+                # Catch abuses when used with a variable and not a literal
+                bad = 'LOG.%s(%s(msg))' % (log, hint)
+                self.assertEqual(
+                    1, len(list(tc.no_translate_logs(bad, 'f'))))
+                # Do not do validations in tests
+                ok = 'LOG.%s(_("OK - unit tests"))' % log
+                self.assertEqual(
+                    0, len(list(tc.no_translate_logs(ok, 'f/tests/f'))))
 
     def test_check_log_warn_deprecated(self):
-        bad = "LOG.warn(_LW('i am deprecated!'))"
-        good = "LOG.warning(_LW('zlatan is the best'))"
+        bad = "LOG.warn('i am deprecated!')"
+        good = "LOG.warning('zlatan is the best')"
         f = tc.check_log_warn_deprecated
         self.assertLineFails(f, bad, '')
         self.assertLinePasses(f, good, '')
@@ -182,34 +135,55 @@ class HackingTestCase(base.BaseTestCase):
         self.assertLinePasses(f, "raise KeyError('Error text')",
                               'neutron_lib/tests/unit/mytest.py')
 
-    def test_check_delayed_string_interpolation(self):
-        dummy_noqa = re.search('a', 'a')
-        f = tc.check_delayed_string_interpolation
+    def test_check_eventlet_imports(self):
+        f = checks.check_no_eventlet_imports
+        self.assertLineFails(f, "import eventlet")
+        self.assertLineFails(f, "import eventlet.timeout")
+        self.assertLineFails(f, "from eventlet import timeout")
+        self.assertLineFails(f, "from eventlet.timeout import Timeout")
+        self.assertLineFails(f, "from eventlet.timeout import (Timeout, X)")
+        self.assertLinePasses(f, "import is.not.eventlet")
+        self.assertLinePasses(f, "from is.not.eventlet")
+        self.assertLinePasses(f, "from mymod import eventlet")
+        self.assertLinePasses(f, "from mymod.eventlet import amod")
+        self.assertLinePasses(f, 'print("eventlet not here")')
+        self.assertLinePasses(f, 'print("eventlet.timeout")')
+        self.assertLinePasses(f, "from mymod.timeout import (eventlet, X)")
 
-        # In 'logical_line', Contents of strings replaced with
-        # "xxx" of same length.
-        self.assertLineFails(f, 'LOG.error(_LE("xxxxxxxxxxxxxxx") % value)',
-                             'neutron_lib/db/utils.py', None)
-        self.assertLineFails(f, "LOG.warning(msg % 'xxxxx')",
-                             'neutron_lib/db/utils.py', None)
-        self.assertLineFails(f, "LOG.info(_LI('xxxxxxxxxxxxxxxxxxxxxxxxx"
-                                "xxxxxxxxx') % {'xx': v1, 'xx': v2})",
-                             'neutron_lib/db/utils.py', None)
-
-        self.assertLinePasses(
-            f, 'LOG.error(_LE("xxxxxxxxxxxxxxxxxx"), value)',
-            'neutron_lib/db/utils.py', None)
-        self.assertLinePasses(f, "LOG.warning(msg, 'xxxxx')",
-                              'neutron_lib/db/utils.py', None)
-        self.assertLinePasses(f, "LOG.info(_LI('xxxxxxxxxxxxxxxxxxxxxxxx"
-                                 "xxxxxxxxx'), {'xx': v1, 'xx': v2})",
-                              'neutron_lib/db/utils.py', None)
-
-        # check a file in neutron_lib/tests
-        self.assertLinePasses(f, 'LOG.error(_LE("xxxxxxxxxxxxxxx") % value)',
-                              'neutron_lib/tests/unit/test_neutron_lib.py',
-                              None)
-        # check code including 'noqa'
-        self.assertLinePasses(f, 'LOG.error(_LE("xxxxxxxxxxxxxxx") % value)',
-                              'neutron_lib/db/utils.py',
-                              dummy_noqa)
+    def test_assert_equal_none(self):
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual(A, None)"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual(A, None)  # Comment"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual(None, A)"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual(None, A)  # Comment"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual((None, None), A)"))), 0)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual((None, None), A)  # Comment"))), 0)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual(A, (None, None))"))), 0)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "self.assertEqual(A, (None, None))  # Comment"))), 0)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot(A, None)"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot(A, None)  # Comment"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot(None, A)"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot(None, A)  # Comment"))), 1)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot((None, None), A)"))), 0)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot((None, None), A)  # Comment"))), 0)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot(A, (None, None))"))), 0)
+        self.assertEqual(len(list(checks.assert_equal_none(
+            "assertIsNot(A, (None, None))  # Comment"))), 0)
+        self.assertEqual(
+            len(list(checks.assert_equal_none("self.assertIsNone(A)"))), 0)
+        self.assertEqual(
+            len(list(checks.assert_equal_none("self.assertIsNotNone(A)"))), 0)
