@@ -16,11 +16,16 @@ import decimal
 import random
 import weakref
 
-import six
+from oslo_log import log as logging
+from oslo_utils import timeutils
+from oslo_utils import uuidutils
 
 from neutron_lib._i18n import _
 
 
+LOG = logging.getLogger(__name__)
+
+
 def parse_mappings(mapping_list, unique_values=True, unique_keys=True):
     """Parse a list of mapping strings into a dictionary.
 
@@ -130,8 +135,8 @@ def diff_list_of_dict(old_list, new_list):
     :returns: A tuple where the first item is a list of the added dicts in
         the diff and the second item is the removed dicts.
     """
-    new_set = set([dict2str(l) for l in new_list])
-    old_set = set([dict2str(l) for l in old_list])
+    new_set = set([dict2str(i) for i in new_list])
+    old_set = set([dict2str(i) for i in old_list])
     added = new_set - old_set
     removed = old_set - new_set
     return [str2dict(a) for a in added], [str2dict(r) for r in removed]
@@ -174,18 +179,12 @@ def safe_decode_utf8(s):
     :param s: The str to decode.
     :returns: The decoded str.
     """
-    if six.PY3 and isinstance(s, bytes):
+    if isinstance(s, bytes):
         return s.decode('utf-8', 'surrogateescape')
     return s
 
 
-try:
-    # PY3
-    weak_method = weakref.WeakMethod
-except AttributeError:
-    # PY2
-    import weakrefmethod
-    weak_method = weakrefmethod.WeakMethod
+weak_method = weakref.WeakMethod
 
 
 def make_weak_ref(f):
@@ -206,3 +205,17 @@ def resolve_ref(ref):
     if isinstance(ref, weakref.ref):
         ref = ref()
     return ref
+
+
+def timecost(f):
+    call_id = uuidutils.generate_uuid()
+    message_base = ("Time-cost: call %(call_id)s function %(fname)s ") % {
+                    "call_id": call_id, "fname": f.__name__}
+    end_message = (message_base + "took %(seconds).3fs seconds to run")
+
+    @timeutils.time_it(LOG, message=end_message, min_duration=None)
+    def wrapper(*args, **kwargs):
+        LOG.debug(message_base + "start")
+        ret = f(*args, **kwargs)
+        return ret
+    return wrapper
